/*
 * cdc.c
 *
 * By Ross Ridge
 * Public Domain
 *
 * Changes the comments and MRs of a delta.
 *
 */

#include "mysc.h"
#include "version.h"
#include "getopt.h"
#include "fileiter.h"
#include "sccsfile.h"
#include "sf-chkmr.h"

char const main_sccs_id[] = "@(#) MySC cdc.c 1.7 98/01/27 04:33:18";

void
usage() {
	fprintf(stderr, msgptr(24, 
"usage: %s [-MYV] [-m MRs] [-y comments] -r SID file ...\n"),
		prg_name);
}

int
main(int argc, char **argv) {
	int c;
	sid rid = NULL;
	string mrs;
	string comments;
	int return_status = 0;
	
	if (argc > 0) {
		set_prg_name(argv[0]);
	} else {
		set_prg_name("cdc");
	}

	setlocale(LC_ALL, "");
	init_msgdb();

	class getopt opts(argc, argv, "r:m?My?YV");
	for(c = opts.next(); c != getopt::END_OF_ARGUMENTS; c = opts.next()) {
		switch (c) {
		default:
			quit(-2, msgptr(10, "Unsupported option: '%c'"), c);

		case 'r':
			rid = sid(opts.getarg());
			if (!rid.valid()) {
				quit(-2, msgptr(11, "Invalid SID: '%s'"),
				     opts.getarg());
			}
			break;

		case 'm':
			mrs = opts.getarg();
			if (mrs == NULL) {
		case 'M':
				mrs = "";
			}
			break;

		case 'y':
			comments = opts.getarg();
			if (comments == NULL) {
		case 'Y':
				comments = "";
			}
			break;

		case 'V':
			version();
			break;
		}
	}

	if (rid == NULL) {
		quit(-2, msgptr(25, "A SID must be specified on the"
				" command line."));
	}


	list<string> mr_list, comment_list;
	int first = 1;
	int tossed_privileges = 0;

	sccs_file_iterator iter(argc, argv, opts.get_index());
	if (iter.length_zero()) {
		quit(-2, msgptr(16, "No SCCS file specified."));
	}

	for(; iter != sccs_file_iterator(); ++iter) {
		if (tossed_privileges) {
			restore_privileges();
			tossed_privileges = 0;
		}

		sccs_name &name = *iter;
		sccs_file file(name, sccs_file::UPDATE);
		if (file.failed()) {
			return_status = 1;
			continue;
		}

		if (first) {
			if (stdin_is_a_tty()) {
				if (mrs == NULL && file.mr_required()) {
					mrs = prompt_user(msgptr(21, "MRs? "));
				}
				if (comments == NULL) {
					comments = prompt_user(
						msgptr(26, "comments? "));
				}
			}
			mr_list = split_mrs(mrs);
			comment_list = split_comments(comments);
			first = 0;
		}

		if (file.mr_required() && mr_list.length() != 0) {
			if (file.check_mrs(mr_list)) {
				error(msgptr(23, "%s: Invalid MR number(s)."),
				      (char const *) name);
				return_status = 1;
				continue;
			}
		}

		if (!file.is_delta_creator(get_user_name(), rid)) {
			give_up_privileges();
			tossed_privileges = 1;
		}

		if (file.cdc(rid, mr_list, comment_list) == -1) {
			return_status = 1;
			continue;
		}

		if (file.update() == -1) {
			return_status = 1;
			continue;
		}
	}

	return return_status;
}

/* Local variables: */
/* mode: c++ */
/* End: */
