/*
 * fileiter.c
 *
 * By Ross Ridge
 * Public Domain
 *
 * Members of the class fileiter.
 *
 */

#include "mysc.h"
#include "sccsname.h"
#include "fileiter.h"
#include "linebuf.h"

#ifdef CONFIG_SCCS_IDS
static char const sccs_id[] = "@(#) MySC fileiter.c 1.8 98/03/12 17:04:12";
#endif

static int
read_directory(string arg, list<string> &files) {
	char const *dirname = arg;
#ifndef CONFIG_NO_DIRECTORY
	if (dirname[0] != '\0') {
		DIR *dir = opendir((char *) dirname);
		if (dir != NULL) {
			char const *slash = NULL;
			int len = strlen(dirname);
#ifdef CONFIG_MSDOS_FILES
			if (dirname[len - 1] != '/' 
			    && dirname[len - 1] != '\\') {
				if (strchr(dirname, '/') == NULL) {
					slash = "\\";
				} else {
					slash = "/";
				}
			}
#else
			if (dirname[len - 1] != '/') {
				slash = "/";
			}
#endif
			string dirbase(dirname, slash);
			struct dirent *dent = readdir(dir);
			files = NULL;
			while(dent != NULL) {
				string name = string(dirbase, dent->d_name);
				if (sccs_name::valid_filename(name)
				    && is_readable(name)) {
					files.add(name);
				}
				dent = readdir(dir);
			}
			
			closedir(dir);
			return 0;
		}
	}
#endif /* CONFIG_NO_DIRECTORY */
	return -1;
}

char const *
sccs_file_iterator::get_next_name() {
	if (source == STDIN || source == DIRECTORY) {
		if (pos < files.length()) {
			return files[pos++];
		}
		source = ARGS;
	}

	assert(source == ARGS);

	if (argc <= 0) {
		return NULL;
	} else {
		argc--;
		char *tmp = *argv++;
		if (strcmp(tmp, "-") == 0) {
			files = NULL;
			class _linebuf linebuf;
			while(!linebuf.read_line(stdin)) {
				int len = strlen(linebuf);
				if (len > 0 && linebuf[len - 1] == '\n') {
					linebuf[len - 1] = '\0';
				}
				files.add((char const *) linebuf);
			}
			source = STDIN;
			pos = 0;
			return get_next_name();
		} else if (read_directory(tmp, files) == 0) {
			source = DIRECTORY;
			pos = 0;
			return get_next_name();
		}
		return tmp;
	}
}
	
void
sccs_file_iterator::operator ++() {
	name = next_name;
	if (name != NULL && !name.valid()) {
		name.make_valid();
	}
	next_name = get_next_name();
}

sccs_file_iterator::sccs_file_iterator(int ac, char **av, int ind)
	: source(ARGS), argv(av + ind), argc(ac - ind), next_name(NULL) {
	/*
	 * Prime the pump and figure out how many file arguments
	 * there are.
	 */
	operator ++();
	operator ++();
	if (name == NULL) {
		length = ZERO;
		return;
	}
	if (next_name == NULL) {
		length = ONE;
		return;
	}
	length = MANY;
}

					
/* Local variables: */
/* mode: c++ */
/* End: */
