/*
 * fileiter.h
 *
 * By Ross Ridge
 * Public Domain
 *
 * Defines the class sccs_file_iter.
 *
 * @(#) MySC fileiter.h 1.9 98/01/29 00:17:04
 *
 */

#ifndef __FILEITER_H__
#define __FILEITER_H__

#include "list.h"
#include "sccsname.h"

#ifndef CONFIG_NO_DIRECTORY
#include <dirent.h>
#undef DIRECTORY
#endif

/* This class is used to iterate over the list of SCCS files as
   specified on the command line. */

class sccs_file_iterator {
public:
	enum sources { ARGS, STDIN, DIRECTORY };
	enum lengths { ZERO, ONE, MANY };

private:
	enum sources source;
	enum lengths length;

	char **argv;
	int argc;

	list<string> files;
	int pos;

	sccs_name name;
	string next_name;

	char const *get_next_name();

	sccs_file_iterator(sccs_file_iterator const &); /* undefined */
	void operator =(sccs_file_iterator const &); /* undefined */

public:
	sccs_file_iterator(int ac, char **av, int ind = 1);
	sccs_file_iterator(): source(ARGS), argc(0), name() {}

	void operator ++();
	sccs_name &operator *() { assert(name != NULL); return name; }

	int
	operator ==(sccs_file_iterator const &it) const {
		return name == NULL && it.name == NULL;
	}

	int
	operator !=(sccs_file_iterator const &it) const {
		return !operator==(it);
	}

	enum sources is_using() { return source; }	
	int length_zero() { return length == ZERO; }
	int length_many() { return length == MANY; }

	~sccs_file_iterator() {}
};

#endif /* __FILEITER_H__ */

/* Local variables: */
/* mode: c++ */
/* End: */
