/*
 * list.h
 *
 * By Ross Ridge
 * Public Domain
 *
 * Defines the template list.
 *
 * @(#) MySC list.h 1.7 98/04/12 20:00:19
 *
 */
 
#ifndef __LIST_H__
#define __LIST_H__


template <class TYPE>
class list {
protected:
	TYPE *array;
	int len;
	int left;

	void destroy();
	void copy(list const &l);
	void grow(int new_left, int offset = 0);

public:
	list(): array((TYPE *) NULL), len(0), left(0) {}

	list(list const &l) { 
		copy(l);
	}
	
	list &operator =(list const &l);

	void
	operator =(void *p) {
		assert(p == NULL);
		destroy();
		len = 0;
		left = 0;
	}

	int add(TYPE const &ent);

	int length() const { return len; }

#if 0
	int extend(int size);
#endif
	TYPE const &
	index(int i) const {
		assert(i >= 0 && i < len);
		return array[i];
	}

	TYPE const &
	operator [](int i) const {
		return index(i);
	}

	TYPE &
	select(int i) {
		assert(i >= 0 && i < len);
		return array[i];
	}

	void
	qsort(int (*compare_fn)(void const *, void const *)) {
		::qsort((void *) array, len, sizeof(TYPE), compare_fn);
	}

	~list() { destroy(); }
};

#if 0
template <class TYPE>
class auto_free_list: public list<TYPE> {
public:
	~auto_free_list();
};
#endif

#include "list.c"

/* Appends the contents of one list to another. */

template<class T1, class T2> 
list<T1> &
operator +=(list<T1> &l1, list<T2> const &l2) {
	int i;
	int len = l2.length();
	for(i = 0; i < len; i++) {
		l1.add(l2[i]);
	}
	return l1;
}

/* Deletes the contents of one list from another. */

template<class T1, class T2> 
list<T1> &
operator -=(list<T1> &l1, list<T2> const &l2) {
	int i, j;
	int len1 = l1.length();
	int len2 = l2.length();
	
	if (len2 > 0) {
		list<T1> nl;

		for(i = 0; i < len1; i++) {
			T1 elt = l1[i];
			for(j = 0; j < len2; j++) {
				if (elt == (T1) l2[j]) {
					break;
				}
			}
			if (j == len2) {
				nl.add(elt);
			}
		}

		l1 = nl;
	}

	return l1;
}


#endif /* __LIST_H__ */

/* Local variables: */
/* mode: c++ */
/* End: */
