/*
 * quit.h
 *
 * By Ross Ridge
 * Public Domain
 *
 * @(#) MySC quit.h 1.10 98/01/06 04:23:41
 *
 */

#ifndef __QUIT_H__
#define __QUIT_H__

#include <setjmp.h>

class cleanup {
	static class cleanup *head;
	static int running;
	static int all_disabled;
	static int in_child_flag;

	class cleanup *next;
	
protected:
	cleanup();
	virtual int do_cleanup() = 0;

#ifdef __GNUC__
	virtual /* not needed but it gets rid of an anoying warning */
#endif
	~cleanup();

public:
	static void run_cleanups();
	static int active() { return running; }
	static void disable_all() { all_disabled++; }
	static void set_in_child() { in_child_flag = 1; disable_all(); }
	static int in_child() { return in_child_flag; }
};

extern char const *prg_name;

void set_prg_name(char const *name);

#ifdef __GNUC__
void quit(int err, char const *fmt, ...) NORETURN
	__attribute__((format(printf, 2, 3)));
void quit(int err, int ret, char const *fmt, ...) NORETURN
	__attribute__((format(printf, 3, 4)));
void error(char const *fmt, ...)
	__attribute__((format(printf, 1, 2)));
void error(int err, char const *fmt, ...)
	__attribute__((format(printf, 2, 3)));
#else
void quit(int err, char const *fmt, ...) NORETURN;
void quit(int err, int ret, char const *fmt, ...) NORETURN;
void error(char const *fmt, ...);
void error(int err, char const *fmt, ...);
#endif

void nomem() NORETURN;
void assert_failed(char const *file, int line, char const *test) NORETURN;

#define assert(test) ((test) ? (void) 0					\
		             : assert_failed(__FILE__, __LINE__, #test))

extern void usage();

#endif /* __QUIT_H__ */

/* Local variables: */
/* mode: c++ */
/* End: */
