/*
 * rmdel.c
 *
 * By Ross Ridge
 * Public Domain
 *
 * Removes a delta from an SCCS file.
 *
 */

#include "mysc.h"
#include "version.h"
#include "fileiter.h"
#include "sccsfile.h"
#include "pfile.h"
#include "getopt.h"

char const main_sccs_id[] = "@(#) MySC rmdel.c 1.6 98/01/27 04:33:23";

void
usage() {
	fprintf(stderr, msgptr(78, 
"usage: %s [-V] -r SID file ...\n"),
		prg_name);
}

int
main(int argc, char **argv) {
	int c;
	sid rid = NULL;
	int return_status = 0;

	if (argc > 0) {
		set_prg_name(argv[0]);
	} else {
		set_prg_name("rmdel");
	}

	setlocale(LC_ALL, "");
	init_msgdb();

	class getopt opts(argc, argv, "r:V");
	for(c = opts.next(); c != getopt::END_OF_ARGUMENTS; c = opts.next()) {
		switch (c) {
#if 0		
		default:
			quit(-2, msgptr(10, "Unsupported option: '%c'"), c);
#endif

		case 'r':
			rid = sid(opts.getarg());
			if (!rid.valid()) {
				quit(-2, msgptr(11, "Invalid SID: '%s'"),
				     opts.getarg());
			}
			break;

		case 'V':
			version();
			break;
		}
	}

	if (rid == NULL) {
		quit(-2, msgptr(25, "A SID must be specified on the"
				" command line."));
	}

	int tossed_privileges = 0;

	sccs_file_iterator iter(argc, argv, opts.get_index());
	if (iter.length_zero()) {
		quit(-2, msgptr(16, "No SCCS file specified."));
	}

	for(; iter != sccs_file_iterator(); ++iter) {
		if (tossed_privileges) {
			restore_privileges();
			tossed_privileges = 0;
		}

		sccs_name &name = *iter;

		sccs_file file(name, sccs_file::UPDATE);
		if (file.failed()) {
			return_status = 1;
			continue;
		}

		sccs_pfile pfile(name, sccs_pfile::READ);
		if (pfile.failed()) {
			return_status = 1;
			continue;
		}

		int found = 0;

		pfile.rewind();
		while(pfile.next()) {
			if (pfile->got == rid) {
				found = 1;
			}
		}
	
		if (found) {
			error(msgptr(79, "%s: Requested SID is locked"
				     " for editing."),
			      (char const *) name);
			return_status = 1;
			continue;
		}

		if (!file.is_delta_creator(get_user_name(), rid)) {
			give_up_privileges();
			tossed_privileges = 1;
		}

		if (file.rmdel(rid) == -1) {
			return_status = 1;
		}
	}

	return return_status;
}

/* Local variables: */
/* mode: c++ */
/* End: */
