/*
 * run.c
 *
 * By Ross Ridge
 * Public Domain
 *
 * Routines for running programmes.
 *
 */

#include "mysc.h"
#include "run.h"
#include "list.h"
#include "sysdep.h"

#ifdef CONFIG_SCCS_IDS
static char const sccs_id[] = "@(#) MySC run.c 1.4 98/06/06 16:25:55";
#endif

/* Runs a programme and returns its exit status. */

int
run(char const *prg, list<string> const &args) {
	int i;
	int len = args.length();

#if defined(CONFIG_NO_FORK) && defined(CONFIG_NO_SPAWN)

	int cmdlen = strlen(prg) + 1;
	
	for(i = 0; i < len; i++) {
		cmdlen += strlen(args[i]) + 1;
	}

	char *s = (char *) xmalloc(cmdlen + 1);

	strcpy(s, prg);
	
	for(i = 0; i < len; i++) {
		strcat(s, " ");
		strcat(s, args[i]);
	}

	int ret = system(s);

#ifdef CONFIG_DJGPP
	if (ret == -1) {
#else
	if (ret != 0) {
#endif
		error(errno, msgptr(80, "system(\"%s\") failed."), s);
		return -1;
	}

	free(s);

#else /* defined(CONFIG_NO_FORK) && defined(CONFIG_NO_SPAWN) */

	char const **argv = (char const **) xmalloc((len + 2) 
						    * sizeof(char const **));

	argv[0] = prg;

	for(i = 0; i < len; i++) {
		argv[i + 1] = args[i];
	}
	
	argv[i + 1] = NULL;

#ifdef CONFIG_NO_FORK

	int ret = spawnvp(P_WAIT, (char *) prg, (char **) argv);
	if (ret == -1) {
		error(errno, msgptr(81, "spawnvp(\"%s\") failed."), prg);
		return -1;
	}

#else /* CONFIG_NO_FORK */

	int pid = fork(); 
	if (pid < 0) {
		error(errno, msgptr(70, "fork() failed."));
		return -1;
	}

	if (pid == 0) {
		cleanup::set_in_child();
		execvp(prg, (char **) argv);
		error(errno, msgptr(82, "execvp(\"%s\") failed."), prg);
		return -1;
	}

	int ret;
	int r = wait(&ret);
	while(r != pid) {
		if (r == -1 && errno != EINTR) {
			error(errno, msgptr(68, "wait() failed."));
			return -1;
		}
		r = wait(&ret);
	}

#endif /* CONFIG_NO_FORK */

	free(argv);

#endif /* defined(CONFIG_NO_FORK) && defined(CONFIG_NO_SPAWN) */

	return ret;
}


/* Runs a programme to checks MRs. */

int
run_mr_checker(char const *prg, char const *arg1, list<string> mrs) {
	list<string> args;

	args.add(arg1);
	args += mrs;

	return run(prg, args);
}

/* Local variables: */
/* mode: c++ */
/* End: */
