/*
 * sf-get2.c 
 *
 * By Ross Ridge
 * Public Domain
 *
 * Members of the class sccs_file only used by get.
 *
 */

#include "mysc.h"
#include "sccsfile.h"
#include "pfile.h"
#include "seqstate.h"

#include <ctype.h>

#ifdef CONFIG_SCCS_IDS
static char const sccs_id[] = "@(#) MySC sf-get2.c 1.8 98/06/06 16:26:06";
#endif

sid
sccs_file::find_latest_sid(sid requested) const {
	if (requested == NULL) {
		requested = flags.default_sid;
	}

	delta_iterator iter(delta_table);

	if (requested.release_only()) {
		/*
		 * Find the first first sid with the highest release number
		 * lower than the requested release.
		 */
		release req_rel = requested;
		sid best = NULL;
		while(iter.next()) {
			release rel = (release) iter->id;
			if (rel == req_rel) {
				return iter->id;
			}
			if (rel < req_rel
			    && (best == NULL || rel > (release) best)) {
				best = iter->id;
			}
		}
		return best;
	} else {
		while(iter.next()) {
			if (requested.partial_match(iter->id)) {
				return iter->id;
			}
		}
	}
	
	return (sid) NULL;
}

/* Returns the SID of the new delta to be created. */

sid
sccs_file::find_next_sid(sid requested, sid got, int branch,
			 sccs_pfile &pfile) const {
	branch = branch && flags.branch;

	if (!branch
	    && !got.is_trunk_successor(delta_table.highest_release()))  {
		if (requested > delta_table.highest_release()) {
			assert(requested.release_only());
			return requested.successor();
		}

		sid next = got.successor();

		if (!pfile.is_to_be_created(next)
		    && delta_table.find(next) == NULL) {
			return next;
		}
	}

	sid highest = got;

	delta_iterator iter(delta_table);
	while(iter.next()) {
		sid const &id = iter->id;

		if (id.branch_greater_than(highest)) {
			highest = id;
		}
	}

	pfile.rewind();
	while(pfile.next()) {
		sid const &id = pfile->delta;
		
		if (id.branch_greater_than(highest)) {
			highest = id;
		}
	}

	return highest.next_branch();
}


/* Quits if the user isn't authorized to make deltas, if the release
   requested is locked or if the requested SID has an edit lock and
   the joint edit flag isn't set. */

int
sccs_file::test_locks(sid got, sccs_pfile &pfile) const {
	int i;
	int len;

	char const *user = get_user_name();

	len = users.length();
	if (len != 0) {
		int found = 0;

		for(i = 0; i < len; i++) {
			char const *s = users[i];
			char c = s[0];

			if (c == '!') {
				s++;
				if (isdigit(c)) {
					if (user_is_group_member(atoi(s))) {
						found = 0;
						break;
					}
				} else if (strcmp(s, user) == 0) {
					found = 0;
					break;
				}
			} else {
				if (isdigit(c)) {
					if (user_is_group_member(atoi(s))) {
						found = 1;
					}
				} else if (strcmp(s, user) == 0) {
					found = 1;
					break;
				}
			}
		}
		
		if (!found) {
			error(msgptr(141, "%s: You are not authorized to"
				     " make deltas."),
			      (char const *) name);
			return -1;
		}
	}

	if (flags.all_locked 
	    || (flags.floor != NULL && flags.floor > got)
	    || (flags.ceiling != NULL && flags.ceiling < got)
	    || flags.locked.member(got)) {
		error(msgptr(142, "%s: Requested release is locked."),
		      (char const *) name);
		return -1;
	}
	
	if (pfile.is_locked(got) && !flags.joint_edit) {
		error(msgptr(143, "%s: Requested SID locked by '%s'.\n"),
		      (char const *) name, (char const *) pfile->user);
		return -1;
	}

	return 0;
}


/* Local variables: */
/* mode: c++ */
/* End: */
