/* 
 * what.c
 *
 * By Ross Ridge
 * Public Domain
 *
 * Prints out SCCS identification keywords marked with "@(#)" in files.
 *
 */

#include "mysc.h"
#include "version.h"
#include "quit.h"
#include "getopt.h"
#include "sysdep.h"

char const main_sccs_id[] = "@(#) MySC what.c 1.10 98/03/12 17:04:09";

/* Inline fuctions for reading files with stdio. */

void
usage() {
	fprintf(stderr, msgptr(174, "usage: %s [-sV] file ...\n"), prg_name);
}


/* Print what's found after a "@(#)" in a file */

inline int
at_end(int c) {
	switch (c) {
	case '"':
	case '>':
	case '\n':
	case '\r':
	case '\\':
	case '\0':
		return 1;
	}
	return 0;
}

inline int
print_what(FILE *f) {
	putchar('\t');
	while(1) {
		int c = getc(f);
		if (c == EOF) {
			return -1;
		}
		if (at_end(c)) {
			putchar('\n');
			return 0;
		}
		putchar(c);
	}
}

int
main(int argc, char **argv) {
	int one_match = 0;
	int error_status = 0;

	if (argc > 0) {
		set_prg_name(argv[0]);
	} else {
		set_prg_name("what");
	}

	setlocale(LC_ALL, "");
	init_msgdb();

	int c;
	class getopt opts(argc, argv, "r:snV");
	for(c = opts.next(); c != getopt::END_OF_ARGUMENTS; c = opts.next()) {
		switch (c) {
		case 's':
			one_match = 1;
			break;

		case 'V':
			version();
		}
	}

	int arg;
	for(arg = opts.get_index(); arg < argc; arg++) {
		int pos = 0;
		FILE *f = fopen(argv[arg], "rb");
		if (f == NULL) {
			error(errno, msgptr(175, "%s: Can't open file."),
			      argv[arg]);
			error_status = 1;
			continue;
		}
		
		printf("%s:\n", argv[arg]);

		while(1) {
			char const magic[4] = { '@', '(', '#', ')' };
			int c;

			c = getc(f);
			if (c == EOF) {
				break;
			}

			if (c == magic[pos]) {
				pos++;
				if (pos == 4) {
					if (print_what(f) == -1 || one_match) {
						break;
					}
				}
			} else {
				pos = 0;
			}
		}
						
		if (ferror(f)) {
			error(errno, msgptr(59, "%s: Read error."), argv[arg]);
			error_status = 1;
		}
				
		putchar('\n');
		fclose(f);
	}

	return error_status;
}

/* Local variables: */
/* mode: c++ */
/* End: */
